@@ -125,11 +125,20 @@ int inet_ptom (int af, const char *src, unsigned int *dst)
  */
 const char *inet_mtop (int af, unsigned int src, char *dst, socklen_t len)
 {
-    struct in_addr addr;
-    
-    inet_mton(AF_INET, src, &addr);
-    
-    return inet_ntop (af, &addr, dst, len);
+    if (af == AF_INET) {
+        struct in_addr addr;
+
+        inet_mton(af, src, &addr);
+        return inet_ntop(af, &addr, dst, len);
+    } else if (af == AF_INET6) {
+        struct in6_addr *addr6;
+
+        inet_mton(af, src, &addr6);
+        return inet_ntop(af, &addr6, dst, len);
+    } else {
+        di_warning("Unknown address family passed to inet_mtop: %i", af);
+        return NULL;
+    }
 }
 
 /* convert a mask length (eg 24) in +src+ into the struct in_addr it corresponds
@@ -1680,8 +1689,22 @@ int netcfg_gateway_reachable(const struct netcfg_interface *interface)
     inet_pton(interface->address_family, interface->gateway, &gw_addr);
 
     if (interface->address_family == AF_INET) {
+        if (interface->masklen == 32) {
+            /* Special case for single-address networks.  We'll handle
+             * routing manually in this case.
+             */
+            return 1;
+        }
+
         return (gw_addr.in4.s_addr && ((gw_addr.in4.s_addr & mask.in4.s_addr) == net.in4.s_addr));
     } else if (interface->address_family == AF_INET6) {
+        if (interface->masklen == 128) {
+            /* Special case for single-address networks.  We'll handle
+             * routing manually in this case.
+             */
+            return 1;
+        }
+
         if ((ntohs(gw_addr.in6.s6_addr16[0]) & 0xffc0) == (0xfe80 & 0xffc0)) {
             return 1;
         }
